/* C64/C128 SuperCPU accelerator test code. */

#define ACC_DETECT      detect_scpu
#define ACC_GET_SPEED   get_scpu_speed
#define ACC_SET_SPEED   set_scpu_speed
#define ACC_NAME        "SuperCPU"

#include "turbo-test.c"
