;/*****************************************************************************/
;/*                                                                           */
;/*                                signal.inc                                 */
;/*                                                                           */
;/*                        Signal handling definitions                        */
;/*                                                                           */
;/*                                                                           */
;/*                                                                           */
;/* (C) 2002      Ullrich von Bassewitz                                       */
;/*               Wacholderweg 14                                             */
;/*               D-70597 Stuttgart                                           */
;/* EMail:        uz@musoftware.de                                            */
;/*                                                                           */
;/*                                                                           */
;/* This software is provided 'as-is', without any expressed or implied       */
;/* warranty.  In no event will the authors be held liable for any damages    */
;/* arising from the use of this software.                                    */
;/*                                                                           */
;/* Permission is granted to anyone to use this software for any purpose,     */
;/* including commercial applications, and to alter it and redistribute it    */
;/* freely, subject to the following restrictions:                            */
;/*                                                                           */
;/* 1. The origin of this software must not be misrepresented; you must not   */
;/*    claim that you wrote the original software. If you use this software   */
;/*    in a product, an acknowledgment in the product documentation would be  */
;/*    appreciated but is not required.                                       */
;/* 2. Altered source versions must be plainly marked as such, and must not   */
;/*    be misrepresented as being the original software.                      */
;/* 3. This notice may not be removed or altered from any source              */
;/*    distribution.                                                          */
;/*                                                                           */
;/*****************************************************************************/



; Standard signal handling functions
SIG_ERR         = $0000

; Signal numbers
SIGABRT         = 0
SIGFPE          = 1
SIGILL          = 2
SIGINT          = 3
SIGSEGV         = 4
SIGTERM         = 5
SIGCOUNT        = 6             ; Number of signals

; Table with signal handlers (asm code only)
.global         sigtable

; Function declarations
.global         __sig_ign
.global         __sig_dfl
.global         _signal
.global         _raise


