;
; Maciej 'YTM/Alliance' Witkowiak
;
; 29.10.99

; void RecoverRectangle (void);
            
            .export _RecoverRectangle

            .include "jumptab.inc"
            
_RecoverRectangle = RecoverRectangle

