; Dummy version, there is no visible cursor in the default CONIO screen
;
; 28-May-2014, Christian Groessler <chris@groessler.org>

        .export setcursor

.proc   setcursor

        rts

.endproc
