/* C64 Chameleon accelerator test code. */

#define ACC_DETECT      detect_chameleon
#define ACC_GET_SPEED   get_chameleon_speed
#define ACC_SET_SPEED   set_chameleon_speed
#define ACC_NAME        "Chameleon cartridge"

#include "turbo-test.c"
