;
; Maciej 'YTM/Alliance' Witkowiak
;
; 21.12.99

; void ImprintRectangle (void);
            
            .export _ImprintRectangle

            .include "jumptab.inc"
            
_ImprintRectangle = ImprintRectangle

