
;GEOS Printer Driver Jump Table
;reassembled by Maciej 'YTM/Alliance' Witkowiak
;4-2-99

;jump table
InitForPrint            =       $7900
StartPrint              =       $7903
PrintBuffer             =       $7906
StopPrint               =       $7909
GetDimensions           =       $790c
PrinsASCII              =       $790f
StartASCII              =       $7912
SetNLQ                  =       $7915
