;
; Maciej 'YTM/Alliance' Witkowiak
;
; 29.10.99

; void InvertRectangle  (void);

            .export _InvertRectangle
            
            .include "jumptab.inc"

_InvertRectangle = InvertRectangle

