;
;GEOS Input Driver Jump Tab
;reassembled by Maciej 'YTM/Alliance' Witkowiak
;4-2-99

;for C64
;MOUSE_JMP              =       $fe80
SlowMouse               =       $fe83
UpdateMouse             =       $fe86
SetMouse                =       $fe89

;for C128
;MOUSE_JMP_128          =       $fd00
SlowMouse_128           =       $fd03
UpdateMouse_128         =       $fd06
SetMouse_128            =       $fd09
