;
;GEOS System Jump Table
;reassembled by Maciej 'YTM/Alliance' Witkowiak
;4-2-99

;jump table
InterruptMain           =       $c100
InitProcesses           =       $c103
RestartProcess          =       $c106
EnableProcess           =       $c109
BlockProcess            =       $c10c
UnblockProcess          =       $c10f
FreezeProcess           =       $c112
UnfreezeProcess         =       $c115
HorizontalLine          =       $c118
InvertLine              =       $c11b
RecoverLine             =       $c11e
VerticalLine            =       $c121
Rectangle               =       $c124
FrameRectangle          =       $c127
InvertRectangle         =       $c12a
RecoverRectangle        =       $c12d
DrawLine                =       $c130
DrawPoint               =       $c133
GraphicsString          =       $c136
SetPattern              =       $c139
GetScanLine             =       $c13c
TestPoint               =       $c13f
BitmapUp                =       $c142
PutChar                 =       $c145
PutString               =       $c148
UseSystemFont           =       $c14b
StartMouseMode          =       $c14e
DoMenu                  =       $c151
RecoverMenu             =       $c154
RecoverAllMenus         =       $c157
DoIcons                 =       $c15a
DShiftLeft              =       $c15d
BBMult                  =       $c160
BMult                   =       $c163
DMult                   =       $c166
Ddiv                    =       $c169
DSdiv                   =       $c16c
Dabs                    =       $c16f
Dnegate                 =       $c172
Ddec                    =       $c175
ClearRam                =       $c178
FillRam                 =       $c17b
MoveData                =       $c17e
InitRam                 =       $c181
PutDecimal              =       $c184
GetRandom               =       $c187
MouseUp                 =       $c18a
MouseOff                =       $c18d
DoPreviousMenu          =       $c190
ReDoMenu                =       $c193
GetSerialNumber         =       $c196
Sleep                   =       $c199
ClearMouseMode          =       $c19c
i_Rectangle             =       $c19f
i_FrameRectangle        =       $c1a2
i_RecoverRectangle      =       $c1a5
i_GraphicsString        =       $c1a8
i_BitmapUp              =       $c1ab
i_PutString             =       $c1ae
GetRealSize             =       $c1b1
i_FillRam               =       $c1b4
i_MoveData              =       $c1b7
GetString               =       $c1ba
GotoFirstMenu           =       $c1bd
InitTextPrompt          =       $c1c0
MainLoop                =       $c1c3
DrawSprite              =       $c1c6
GetCharWidth            =       $c1c9
LoadCharSet             =       $c1cc
PosSprite               =       $c1cf
EnablSprite             =       $c1d2
DisablSprite            =       $c1d5
CallRoutine             =       $c1d8
CalcBlksFree            =       $c1db
ChkDkGEOS               =       $c1de
NewDisk                 =       $c1e1
GetBlock                =       $c1e4
PutBlock                =       $c1e7
SetGEOSDisk             =       $c1ea
SaveFile                =       $c1ed
SetGDirEntry            =       $c1f0
BldGDirEntry            =       $c1f3
GetFreeDirBlk           =       $c1f6
WriteFile               =       $c1f9
BlkAlloc                =       $c1fc
ReadFile                =       $c1ff
SmallPutChar            =       $c202
FollowChain             =       $c205
GetFile                 =       $c208
FindFile                =       $c20b
CRC                     =       $c20e
LdFile                  =       $c211
EnterTurbo              =       $c214
LdDeskAcc               =       $c217
ReadBlock               =       $c21a
LdApplic                =       $c21d
WriteBlock              =       $c220
VerWriteBlock           =       $c223
FreeFile                =       $c226
GetFHdrInfo             =       $c229
EnterDeskTop            =       $c22c
StartAppl               =       $c22f
ExitTurbo               =       $c232
PurgeTurbo              =       $c235
DeleteFile              =       $c238
FindFTypes              =       $c23b
RstrAppl                =       $c23e
ToBASIC                 =       $c241
FastDelFile             =       $c244
GetDirHead              =       $c247
PutDirHead              =       $c24a
NxtBlkAlloc             =       $c24d
ImprintRectangle        =       $c250
i_ImprintRectangle      =       $c253
DoDlgBox                =       $c256
RenameFile              =       $c259
InitForIO               =       $c25c
DoneWithIO              =       $c25f
DShiftRight             =       $c262
CopyString              =       $c265
CopyFString             =       $c268
CmpString               =       $c26b
CmpFString              =       $c26e
FirstInit               =       $c271
OpenRecordFile          =       $c274
CloseRecordFile         =       $c277
NextRecord              =       $c27a
PreviousRecord          =       $c27d
PointRecord             =       $c280
DeleteRecord            =       $c283
InsertRecord            =       $c286
AppendRecord            =       $c289
ReadRecord              =       $c28c
WriteRecord             =       $c28f
SetNextFree             =       $c292
UpdateRecordFile        =       $c295
GetPtrCurDkNm           =       $c298
PromptOn                =       $c29b
PromptOff               =       $c29e
OpenDisk                =       $c2a1
DoInlineReturn          =       $c2a4
GetNextChar             =       $c2a7
BitmapClip              =       $c2aa
FindBAMBit              =       $c2ad
SetDevice               =       $c2b0
IsMseInRegion           =       $c2b3
ReadByte                =       $c2b6
FreeBlock               =       $c2b9
ChangeDiskDevice        =       $c2bc
RstrFrmDialogue         =       $c2bf
Panic                   =       $c2c2
BitOtherClip            =       $c2c5
StashRAM                =       $c2c8
FetchRAM                =       $c2cb
SwapRAM                 =       $c2ce
VerifyRAM               =       $c2d1
DoRAMOp                 =       $c2d4

;only in GEOS 128
TempHideMouse           =       $c2d7
SetMousePicture         =       $c2da
SetNewMode              =       $c2dd
NormalizeX              =       $c2e0
MoveBData               =       $c2e3
SwapBData               =       $c2e6
VerifyBData             =       $c2e9
DoBOp                   =       $c2ec
AccessCache             =       $c2ef
HideOnlyMouse           =       $c2f2
SetColorMode            =       $c2f5
ColorCard               =       $c2f8
ColorRectangle          =       $c2fb
