/*
  !!DESCRIPTION!! minimal Program, checks if the Compiler and testsuite framework works
  !!ORIGIN!!      testsuite
  !!LICENCE!!     Public Domain
  !!AUTHOR!!      Groepaz/Hitmen
*/

#include <stdio.h>

int main(void)
{
#if 1
    printf("it works :)\n");
#endif
    return 0;
}
