/* C64 Turbo Master accelerator test code. */

#define ACC_DETECT      detect_turbomaster
#define ACC_GET_SPEED   get_turbomaster_speed
#define ACC_SET_SPEED   set_turbomaster_speed
#define ACC_NAME        "Turbo Master cartridge"

#include "turbo-test.c"
