;
; Oliver Schmidt, 2013-05-31
;

        .export         em_libref, joy_libref, mouse_libref, ser_libref, tgi_libref
        .import         _exit

em_libref       := _exit
joy_libref      := _exit
mouse_libref    := _exit
ser_libref      := _exit
tgi_libref      := _exit
