;
; Maciej 'YTM/Alliance' Witkowiak
;
; 25.12.99

; char RstrFrmDialogue  (void);

            .export _RstrFrmDialogue

            .include "jumptab.inc"
        
_RstrFrmDialogue = RstrFrmDialogue
