;
;GEOS various variables in OS_ROM area
;reassembled by Maciej 'YTM/Alliance' Witkowiak
;4-2-99

bootName                =       $c006
gatewayFlag             =       $c007
version                 =       $c00f
nationality             =       $c010
sysFlgCopy              =       $c012
c128Flag                =       $c013
mp3Flag                 =       $c014
dateCopy                =       $c018

