body {
    font-family:        arial, helvetica, sans-serif;
    font-size:          100%;
    text-align:         justify;
    margin-left:        110px;
    margin-top:         10px;
    margin-right:       30px;
    margin-bottom:      10px;
    background-image:   url(doc.png);
    background-repeat:  repeat-y;
}

h1, h2, h2 a:link, h2 a:active, h2 a:visited {
    font-weight:        bold;
    font-style:         italic;
    text-align:         left;
    color:              #DB3232;
}

h1 {
    font-size:          250%;
    text-shadow:        2px 2px 6px #505050;
    letter-spacing:     2px;
    padding-top:        40px;
}

h2 {
    font-size:          160%;
    text-shadow:        1px 1px 3px #303030;
    letter-spacing:     1px;
    margin-top:         2em;
    margin-bottom:      1em;
}
