/*
** test program for the cprintf() function
** (for the \r and \n special operators)
** CPG
**
** The output generated by this program should look
** like this:
**
** ---- top of screen --------
** 12345
**      67890
**
**
** 67890
**
** hit return to exit....
 .
 .
 .
** ---- bottom of screen -----
*/

#include <stdio.h>
#include <conio.h>

int main(void)
{
        clrscr();
        cprintf("12345\n");
        cprintf("67890\n");
        gotoxy(0,4);
        cprintf("12345\r");
        cprintf("67890\r");
        printf("\n\n");
        printf("hit return to exit....\n");
        fgetc(stdin);
        return(0);
}
