;
; Maciej 'YTM/Alliance' Witkowiak
;
; 30.10.99

; void StartMouseMode (void);

            .export _StartMouseMode

            .include "jumptab.inc"

_StartMouseMode:
        clc
        jmp StartMouseMode