
#include <stdio.h>
#include <stdlib.h>

#define NO_OLD_FUNC_DECL
#define NO_TYPELESS_INT
#define NO_TYPELESS_INT_PTR
#define MAIN_RETURNS_INT
#define NO_IMPLICIT_FUNC_PROTOTYPES
#define NO_FLOATS
#define NO_WCHAR
#define NO_EMPTY_FUNC_ARGS
#define NO_SLOPPY_STRUCT_INIT
#define NO_FUNCS_TAKE_STRUCTS
#define NO_FUNCS_RETURN_STRUCTS
#define CAST_STRUCT_PTR
#define NO_TYPELESS_STRUCT_PTR
#define NO_IMPLICIT_FUNCPTR_CONV
#define SIZEOF_INT_16BIT
#define SIZEOF_LONG_32BIT
#define UNSIGNED_CHARS
#define UNSIGNED_BITFIELDS
