/* C128 in C64 mode accelerator test code. */

#define ACC_DETECT      detect_c128
#define ACC_GET_SPEED   get_c128_speed
#define ACC_SET_SPEED   set_c128_speed
#define ACC_NAME        "C128 CPU"

#include "turbo-test.c"
