;
; Ullrich von Bassewitz, 19.11.2002
;
; PET kernal functions
;

        .include "cbm_kernal.inc"

        .export         CLRCH
        .export         BASIN
        .export         STOP
        .export         GETIN
        .export         CLALL
        .export         UDTIM
