;
;GEOS Disk Driver JumpTab
;Source: The Hitchhiker's Guide To GEOS
;http://lyonlabs.org/commodore/onrequest/geos-manuals/The_Hitchhikers_Guide_to_GEOS.pdf

GetBlock                =       $0300
PutBlock                =       $0303
GetFile                 =       $030f
LdFile                  =       $0312
ReadFile                =       $0315
SaveFile                =       $0318
WriteFile               =       $031b
GetVBM                  =       $031e
PutVBM                  =       $0321
CalcBlksFree            =       $0324
FreeBlock               =       $032a
SetNextFree             =       $032d
AppendRecord            =       $0330
CloseRecordFile         =       $0333
DeleteRecord            =       $0336
NextRecord              =       $0336
InsertRecord            =       $0339
OpenRecordFile          =       $033f
PointRecord             =       $0342
PreviousRecord          =       $0345
ReadRecord              =       $0348
UpdateRecordFile        =       $034b
WriteRecord             =       $034e
BlkAlloc                =       $0351
NxtBlkAlloc             =       $0354
DeleteFile              =       $0357
FreeFile                =       $035a
LdDeskAcc               =       $035d
RstrAppl                =       $0360
FdFTypesInDir           =       $0363
FindFTypes              =       $0366
FindFile                =       $0369
FndFilinDir             =       $036c
BldGDirEntry            =       $036f
GetDirHead              =       $0372
GetFreeDirBlk           =       $0375
PutDirHead              =       $037b
SetGDirEntry            =       $037b
DeleteDir               =       $037e
DownDirectory           =       $0381
GetPathname             =       $0384
MakeSubDir              =       $0387
UpDirectory             =       $038a
LdApplic                =       $038d
GetFHdrInfo             =       $0390
RenameFile              =       $0393
OpenDisk                =       $0399
Get1stDirEntry          =       $039c
GetNxtDirEntry          =       $039c
GoDirectory             =       $039f
ReOpenDisk              =       $03a2
SortAlpha               =       $03a5
SaveFG                  =       $03ab
RecoverFG               =       $03ae
JsrToAux                =       $03b4
UpdateParent            =       $03b7
FreeDir                 =       $03ba
