;
; Maciej 'YTM/Alliance' Witkowiak
;
; 29.10.99

; void FrameRectangle   (char pattern);

            .export _FrameRectangle

            .include "jumptab.inc"
            
_FrameRectangle = FrameRectangle
