/* C64DTV accelerator test code. */

#define ACC_DETECT      detect_c64dtv
#define ACC_GET_SPEED   get_c64dtv_speed
#define ACC_SET_SPEED   set_c64dtv_speed
#define ACC_NAME        "C64DTV"

#include "turbo-test.c"
