/* C65/C64DX in C64 mode accelerator test code. */

#define ACC_DETECT      detect_c65
#define ACC_GET_SPEED   get_c65_speed
#define ACC_SET_SPEED   set_c65_speed
#define ACC_NAME        "C65"

#include "turbo-test.c"
