;
;GEOS variable memory locations sorted by address
;Source: The Hitchhiker's Guide To GEOS
;http://lyonlabs.org/commodore/onrequest/geos-manuals/The_Hitchhikers_Guide_to_GEOS.pdf

r0                      =       $00
r0L                     =       $00
r0H                     =       $01
r1                      =       $02
r1L                     =       $02
r1H                     =       $03
r2                      =       $04
r2L                     =       $04
r2H                     =       $05
r3                      =       $06
r3L                     =       $06
r3H                     =       $07
r4                      =       $08
r4L                     =       $08
r4H                     =       $09
r5                      =       $0a
r5L                     =       $0a
r5H                     =       $0b
r6                      =       $0c
r6L                     =       $0c
r6H                     =       $0d
r7                      =       $0e
r7L                     =       $0e
r7H                     =       $0f
r8                      =       $10
r8L                     =       $10
r8H                     =       $11
r9                      =       $12
r9L                     =       $12
r9H                     =       $13
r10                     =       $14
r10L                    =       $14
r10H                    =       $15
r11                     =       $16
r11L                    =       $16
r11H                    =       $17
r12                     =       $18
r12L                    =       $18
r12H                    =       $19
r13                     =       $1a
r13L                    =       $1a
r13H                    =       $1b
r14                     =       $1c
r14L                    =       $1c
r14H                    =       $1d
r15                     =       $1e
r15L                    =       $1e
r15H                    =       $1f

string                  =       $53
mouseTop                =       $57
mouseBottom             =       $58
mouseLeft               =       $59
mouseRight              =       $5b
mouseOn                 =       $5d
curIndexTable           =       $5e
cardDataPntr            =       $60
returnAddress           =       $64
shiftBuf                =       $70
shiftOutBuf             =       $78
appMain                 =       $0200
intTopVector            =       $0202
intBotVector            =       $0204
mouseVector             =       $0206
keyVector               =       $0208
inputVector             =       $020a
mouseFaultVec           =       $020c
otherPressVec           =       $020e
StringFaultVec          =       $0210
alarmTmtVector          =       $0212
BRKVector               =       $0214
RecoverVector           =       $0216
baselineOffset          =       $0218
curSetWidth             =       $0219
curHeight               =       $021b
currentMode             =       $021c
dispBufferOn            =       $021d
windowTop               =       $021e
leftMargin              =       $0220
rightMargin             =       $0222
selectionFlash          =       $0224
alphaFlag               =       $0225
iconSelFlag             =       $0226
faultData               =       $0227
MenuNumber              =       $0228
mouseXOffset            =       $0229
mouseYOffset            =       $022b
curPattern              =       $022c
stringX                 =       $022e
stringY                 =       $0230
noEraseSprites          =       $0240
mouseXPos               =       $0241
mouseYPos               =       $0243
keyData                 =       $0245
inputData               =       $0247
random                  =       $024c
saveFontTab             =       $024e
dblClickCount           =       $0258
sysDBData               =       $0259
offFlag                 =       $027c
maxMouseSpeed           =       $027d
minMouseSpeed           =       $027e
mouseAccel              =       $027f
firstBoot               =       $0281
dataFileName            =       $02a4
dataDiskName            =       $02b4
MachineType             =       $02c4
intSource               =       $02c6
reqXpos0                =       $0800
reqXpos1                =       $0802
reqXpos2                =       $0804
reqXpos3                =       $0806
reqXpos4                =       $0808
reqXpos5                =       $080a
reqXpos6                =       $080c
reqXpos7                =       $080e
mobenble                =       $0818
mobx2                   =       $0819
moby2                   =       $081a
PrntFileName            =       $08ac
inputDevName            =       $08cc
pressFlag               =       $08fe
curEnable               =       $0951
sspr1back               =       $56d0
sspr2back               =       $5820
sspr3back               =       $5970
sspr4back               =       $5ac0
sspr5back               =       $5c10
sspr6back               =       $5d60
sspr7back               =       $5eb0
backBufPtr              =       $9d7c
softOnes                =       $d000
softZeros               =       $d0e0
curnoby2                =       $d952
curmobx2                =       $d953
curXpos0                =       $d954
curYpos0                =       $d964
backXBufNum             =       $d98c
backYBufNum             =       $d994
sizeFlags               =       $db1c
doRestFlag              =       $db1d
bootName                =       $e003
version                 =       $e00c
nationality             =       $e00d
mouseSave               =       $ee53
year                    =       $f200
month                   =       $f201
day                     =       $f202
hour                    =       $f203
minutes                 =       $f204
seconds                 =       $f205
dlgBoxRamBuf            =       $f381
totNumBlks              =       $f60b
curDrive                =       $f60d
numDrives               =       $f60e
diskOpenFlg             =       $f617
curRecord               =       $f618
usedRecords             =       $f619
fileWritten             =       $f61a
fileSize                =       $f61b
dirBlkno                =       $f620
dirPtr                  =       $f622
indexBlkno              =       $f624
diskBlkBuf              =       $f659
fileHeader              =       $f859
dirEntryBuf             =       $fa59
curDirHead              =       $fa80
DrACurDkNm              =       $faa7
DrBCurDkNm              =       $fab7
DrCCurDkNm              =       $fac7
DrDCurDkNm              =       $fad7
devTabHi                =       $fae7
devTabLo                =       $faeb
devUnitTab              =       $faef
driveType               =       $faf3
drSizeLo                =       $faf7
drSizeHi                =       $fafb
